/*

  "i_apple.inc", Jaime Vives Piqueres, (?)2000.

  object : APPLE

  This file contains two macros, m_apple_t() and apple(). Usually only
  apple() must be called. This macro calls m_apple_t() to obtain a
  the texture for the apple. The macro is neede for this purpouse bcos
  only the upper layer of the texture must rotate to "wrap" radially
  around the object.

  The apple() macro creates the apple shape placing blobs like in a
  SOR, with some randomness. The tricky texture consist on two layers.
  The lower layer gives the basic color of the apple, with a big and
  soft pattern. The second layer, the most interesting, uses a radial
  pattern to "rotate" a slope pattern around y. The slope texture is
  also "rotating" a simple texture around x. This combination looks
  prety close to the real apple texture, at least from a certain
  distance. You can use this method for other similar fruit skins...

*/

/*****************************************************/
/*** macro to randomize the "longitudinal" texture ***
 parameters :
  section counter,
  total sections,
  base color1,
  level color 1,
  base color2,
  level color 2,
  spots color,
  level spots color,
  random seed
******************************************************/
#macro m_apple_t(cnt,n,c1,c1_lv,c2,c2_lv,c3,c3_lv,r_rad)

// declare pigment pattern for the "longitudinal" texture
#local p_slope1=
pigment{
 bumps scale .5
 turbulence .2 lambda 3
 color_map{
  [0.0     Clear]
  [c3_lv-1 Clear] // don't ask why... but it's the only way it works
  [c3_lv   c3]
  [1.0     c3]
 }
 scale <1,18,1>
 scale .4
}

// build texture:
// base layer
texture{
 pigment{
  cylindrical
  turbulence .1
  lambda 2
  color_map{
   [0.0   c1]
   [c1_lv c1]
   [c2_lv c2]
   [1.0   c2]
  }
  scale 5
  translate 2*x
 }
}
// upper "longitudinal" layer
texture{
 pigment{
  slope y
  pigment_map{
   [0.0 p_slope1 translate 10*rand(r_rad) rotate -18*5*z rotate (360*cnt/n)*y]
   [0.1 p_slope1 translate 10*rand(r_rad) rotate -18*4*z rotate (360*cnt/n)*y]
   [0.2 p_slope1 translate 10*rand(r_rad) rotate -18*3*z rotate (360*cnt/n)*y]
   [0.3 p_slope1 translate 10*rand(r_rad) rotate -18*2*z rotate (360*cnt/n)*y]
   [0.4 p_slope1 translate 10*rand(r_rad) rotate -18*1*z rotate (360*cnt/n)*y]
   [0.5 p_slope1 translate 10*rand(r_rad) rotate  18*0*z rotate (360*cnt/n)*y]
   [0.6 p_slope1 translate 10*rand(r_rad) rotate  18*1*z rotate (360*cnt/n)*y]
   [0.7 p_slope1 translate 10*rand(r_rad) rotate  18*2*z rotate (360*cnt/n)*y]
   [0.8 p_slope1 translate 10*rand(r_rad) rotate  18*3*z rotate (360*cnt/n)*y]
   [0.9 p_slope1 translate 10*rand(r_rad) rotate  18*4*z rotate (360*cnt/n)*y]
   [1.0 p_slope1 translate 10*rand(r_rad) rotate  18*5*z rotate (360*cnt/n)*y]
  }
 }
 finish{Phong_Glossy phong .3 phong_size 40 reflection .01}
}
texture{
 pigment{
  cylindrical
  turbulence .1
  lambda 2
  color_map{
   [0.0   c1 filter .5]
   [c1_lv c1 filter .5]
   [c2_lv c2 filter .5]
   [1.0   c2 filter .5]
  }
  scale 5
  translate 2*x
 }
}

#end // of macro m_apple_t()


/**************************/
/*** apple object macro ***
 paramenters :
   random seed,
   base color 1,
   level color 1,
   base color 2,
   level color 2,
   spots color,
   level spots color
***************************/
#macro apple(r_man,c1,c1_lv,c2,c2_lv,c3,c3_lv)

#local angulo=10;
#local num_gajos=360/angulo;
#local cnt_gajos=1;
#local k=2;
#local num_rab=20;
#local cnt_rab=1;

// *** build final texture ***
#local cnt=1;
#local n=17;
#local t_apple=
texture{
 radial
 turbulence .2 lambda 3
 texture_map{
  #declare t_apple_base=m_apple_t(0,n,c1,c1_lv,c2,c2_lv,c3,c3_lv,r_man)
  [0 t_apple_base]
  #while (cnt<=n)
   #declare t_apple_base=m_apple_t(cnt,n,c1,c1_lv,c2,c2_lv,c3,c3_lv,r_man)
   [(cnt/n) t_apple_base]
   #declare cnt=cnt+1;
  #end
 }
 warp{reset_children}
}

// *** build apple ***
union{
blob{
 threshold 0.9
 #while (cnt_gajos<=num_gajos)
  #declare k=2+rand(r_man);
  sphere{<0.5 ,0.8,0>,k*0.4,1 rotate angulo*cnt_gajos*y}
  sphere{<1.25,1,0>,k*0.6,1 rotate (angulo-rand(r_man)*.1+rand(r_man)*.1)*cnt_gajos*y}
  sphere{<2   ,1.1   ,0>,k*0.5 ,1 rotate (angulo-rand(r_man)*.2+rand(r_man)*.2)*cnt_gajos*y}
  sphere{<2.5 ,1.5 ,0>,k*0.51 ,1 rotate (angulo-rand(r_man)*.4+rand(r_man)*.4)*cnt_gajos*y}
  sphere{<3   ,2.2,0>,k*0.52 ,1 rotate (angulo-rand(r_man)*.6+rand(r_man)*.6)*cnt_gajos*y}
  sphere{<3.25,3   ,0>,k*0.53 ,1 rotate (angulo-rand(r_man)*.8+rand(r_man)*.8)*cnt_gajos*y}
  sphere{<3   ,3.75,0>,k*0.7 ,1 rotate (angulo-rand(r_man)+rand(r_man))*cnt_gajos*y}
  sphere{<2.4 ,4.5 ,0>,k*0.8 ,1 rotate (angulo-rand(r_man)+rand(r_man))*cnt_gajos*y}
  sphere{<2 ,4.7,0>,k*0.7 ,1 rotate (angulo-rand(r_man)+rand(r_man))*cnt_gajos*y}
  sphere{<1.6 ,4.9,0>,k*0.6 ,1 rotate (angulo-rand(r_man)+rand(r_man))*cnt_gajos*y}
  sphere{<1.1   ,5   ,0>,k*0.6 ,1 rotate (angulo-rand(r_man)+rand(r_man))*cnt_gajos*y}
  #declare cnt_gajos=cnt_gajos+1;
 #end
 sphere{<0,4.9,0>,k,-1}
 translate -2.5*y
 texture{t_apple translate (-50+100*rand(r_man))*y}
 rotate 360*rand(r_man)*y
}
 cone{<0,0,0>,.1,<-.5+rand(r_man),2*rand(r_man),-.5+rand(r_man)>,.2
  texture{
   pigment{DarkBrown}
   normal{granite 2 scale 12}
  }
  translate 3*y
 }
}


#end
